#
# Makefile for the parser bindings of ABINIT
#

include ../../config.mk

VPATH = $(abinit_srcdir)/bindings/parser \
	$(abinit_srcdir)/src/41_geometry \
	$(abinit_srcdir)/src/56_recipspace

all: libabinit7.a

python: ab7_invars.so

# The building of the library itself.
libabinit_tmpdir = tmp-libabinit7-objects
libabinit7.a: libbindings.a ../../src/libs/libab7_parser.a
	test -e "$(libabinit_tmpdir)" || \
	$(INSTALL) -d -m 755 $(libabinit_tmpdir)
	cd $(libabinit_tmpdir) && $(AR) x ../../../src/libs/libab7_parser.a
	cd $(libabinit_tmpdir) && $(AR) x ../libbindings.a
	$(AR) $(ARFLAGS) libabinit7.a $(libabinit_tmpdir)/*
	$(RANLIB) libabinit7.a
	rm -rf $(libabinit_tmpdir)

ab7_base.o: $(srcdir)/ab7_base.c $(srcdir)/ab7_base.h
	$(CC) $(CPPFLAGS) -I. -I$(srcdir) -I$(abinit_builddir) $(CFLAGS) -c $(abinit_srcdir)/bindings/parser/ab7_base.c

ab7_invars_c.o: $(srcdir)/ab7_invars_c.c $(srcdir)/ab7_invars_c.h $(srcdir)/ab7_invars.h
	$(CC) $(CPPFLAGS) -I. -I$(srcdir) -I$(abinit_builddir) $(CFLAGS) -c $(abinit_srcdir)/bindings/parser/ab7_invars_c.c
ab7_symmetry.o: ab7_symmetry.c ab7_symmetry.h ab7_symmetry.fortran.h
	$(CC) $(CPPFLAGS) -I. -I$(srcdir) -I$(abinit_builddir) -I$(abinit_srcdir)/src/41_geometry $(CFLAGS) -c $(abinit_srcdir)/bindings/parser/ab7_symmetry.c
ab7_kpoints.o: ab7_kpoints.c ab7_kpoints.h ab7_kpoints.fortran.h
	$(CC) $(CPPFLAGS) -I. -I$(srcdir) -I$(abinit_builddir) -I$(abinit_srcdir)/src/56_recipspace $(CFLAGS) -c $(abinit_srcdir)/bindings/parser/ab7_kpoints.c

ab7_dummy.o: $(srcdir)/ab7_dummy.f90
	$(FC) $(FCFLAGS) -c $(abinit_srcdir)/bindings/parser/ab7_dummy.f90

libbindings.a: ab7_base.o ab7_invars_c.o ab7_symmetry.o ab7_kpoints.o ab7_dummy.o
	$(AR) $(ARFLAGS) libbindings.a ab7_base.o ab7_invars_c.o ab7_symmetry.o ab7_kpoints.o ab7_dummy.o
	$(RANLIB) libbindings.a

ab7_invars_py.o: $(srcdir)/ab7_invars_py.h $(srcdir)/ab7_invars.h $(srcdir)/ab7_invars_py.c
	$(CC) $(CPPFLAGS) $(PYTHON_CPPFLAGS) -I. -I$(srcdir) -I$(abinit_builddir) $(CFLAGS) -c $(abinit_srcdir)/bindings/parser/ab7_invars_py.c


ab7_invars.so: ab7_invars_py.o
	$(FC) -shared -o ab7_invars.so ab7_invars_py.o libabinit7.a $(lib_netcdf_libs) $(lib_libxc_libs) $(lib_etsf_io_libs) $(lib_linalg_libs)

clean:
	rm -f *.o *.mod libbindings.a libabinit7.a
	rm -f ab7_invars.so
	rm -f check_invars_c check_invars_f90
	rm -f check_symmetry_f90 check_symmetry_c
	rm -f *.out



# Some additional rules for make check
check_invars_c: $(srcdir)/check_invars_c.c fallbacks.o libabinit7.a
	$(FC) -o check_invars_c -I. -I$(srcdir) $(abinit_srcdir)/bindings/parser/check_invars_c.c libabinit7.a fallbacks.o $(lib_linalg_libs) $(lib_etsf_io_libs) $(lib_netcdf_libs) $(lib_libxc_libs)

check_invars_f90: $(srcdir)/check_invars_f90.f90 fallbacks.o libabinit7.a
	$(FC) -I. -I$(srcdir) $(FCFLAGS) $(FCFLAGS_MODDIR) -o check_invars_f90 $(abinit_srcdir)/bindings/parser/check_invars_f90.f90 libabinit7.a fallbacks.o $(lib_linalg_libs) $(lib_etsf_io_libs) $(lib_netcdf_libs) $(lib_libxc_libs)

fallbacks.o: $(srcdir)/fallbacks.f90
	$(FC) -I. -I$(srcdir) $(FCFLAGS) $(FCFLAGS_MODDIR) -c $(abinit_srcdir)/bindings/parser/fallbacks.f90

check_symmetry_f90: check_symmetry_f90.f90 libabinit7.a fallbacks.o
	$(FC) -I. -I$(srcdir) $(FCFLAGS) $(FCFLAGS_MODDIR) -o check_symmetry_f90 $(abinit_srcdir)/bindings/parser/check_symmetry_f90.f90 -L. -labinis fallbacks.o $(lib_linalg_libs) $(lib_etsf_io_libs) $(lib_netcdf_libs) $(lib_libxc_libs)

check_symmetry_c: $(srcdir)/check_symmetry_c.c libabinit7.a fallbacks.o
	$(FC) -I. -I$(srcdir) $(CFLAGS) -o check_symmetry_c $(abinit_srcdir)/bindings/parser/check_symmetry_c.c -L. -labinis fallbacks.o $(lib_linalg_libs) $(lib_etsf_io_libs) $(lib_netcdf_libs) $(lib_libxc_libs)

check: check_symmetry_f90 check_symmetry_c check_invars_f90 check_invars_c
	./check_symmetry_f90 > check_symmetry_f90.out
	diff $(abinit_srcdir)/bindings/parser/check_symmetry.ref check_symmetry_f90.out
	./check_symmetry_c  > check_symmetry_c.out
	diff $(abinit_srcdir)/bindings/parser/check_symmetry.ref check_symmetry_c.out
	./check_invars_f90 $(abinit_srcdir)/tests/v1/Input/t05.in > check_invars_f90.out
	diff $(abinit_srcdir)/bindings/parser/check_invars.ref check_invars_f90.out
	./check_invars_c $(abinit_srcdir)/tests/v1/Input/t05.in > check_invars_c.out
	diff $(abinit_srcdir)/bindings/parser/check_invars.ref check_invars_c.out
