!------------------------------------!
!---------AVERAGE ROUTINE------------!
!------------------------------------!

subroutine average(data,size_data,average_value)

! input values:  
  integer :: size_data
  real(8),dimension(size_data),intent(in) :: data
 !f2py optional , depend(data) :: size_data=len(data)
  real(8),intent(out) :: average_value
  average_value  = 0.0

  do i=1,size_data
    average_value  = average_value + data(i)
  end do

  average_value = average_value / size_data
  
end subroutine average


!------------------------------------!
!------STANDAR DEVIATION ROUTINE-----!
!------------------------------------!
subroutine standard_deviation(data,average_input,size_data,deviation)
!  input  values:    
  integer  ::  size_data
  real(8), dimension(size_data),intent(in)  ::  data
  real(8), intent(in), optional  ::  average_input  
  !f2py  optional  ,  depend(data)  ::  size_data=len(data)
  real(8),intent(out)  ::  deviation
  real(8) :: average_data = 0

  
  if (average_input/=0) then
    average_data = average_input
  else
    average_data = 0
    call  average(data,size_data,average_data)
  end if

  do  i=1,size_data
    deviation  =  deviation  +  (data(i)  -  average_data)**2
  end  do
    
  deviation  =  deviation/size_data
  deviation  =  deviation**0.5

end subroutine  standard_deviation


!------------------------------------!
!------MEAN SQUARED DISPLACEMENT-----!
!------------------------------------!
subroutine mean_square_displacement(position,indexAtom1,msd,size2,size3,nba1)
  integer :: size2,size3,nba1,nbtime
  !input  values:    
  !f2py  optional  ,  depend(position)    ::  size2=len(position[:,1,1])
  !f2py  optional  ,  depend(position)    ::  size3=len(position[1,:,1])
  !f2py  optional  ,  depend(indexAtom1)  ::  nba1=len(indexAtom1)
  integer, intent(in)  ::  indexAtom1(nba1)
  real, intent(in)  ::  position(size2,size3,3)
  !output 
  real(8),intent(out)  ::  msd(size2)

  !local variable
  real(8), dimension(3)  ::  p1=(/1,1,1/),p2=(/1,1,1/)
  real(8) r
  integer i,k

  nbtime = size2

  msd(:) = 0
  do i=1,nbtime !loop on step
    do k=1,size(indexAtom1)
      p1 = position(1,indexAtom1(k),:)
      p2 = position(i,indexAtom1(k),:) - p1(:)
      r = p2(1)**2+p2(2)**2+p2(3)**2
      msd(i)= msd(i) + r
    end do
  end do
  msd(:) = msd(:)/(size(indexAtom1))
end subroutine mean_square_displacement


!------------------------------------!
!-----PAIR DISTRIBUTION FUNCTION-----!
!------------------------------------!
subroutine pair_distribution(nbtime,position,maxbin,inc,a,b,c,deltaR,rho,indexAtom1,indexAtom2,r,gr,size1,size2,size3,nba1,nba2)
	
  !input
  integer :: size1,size2,size3,nba1,nba2
  integer, intent(in) :: inc,maxbin,nbtime
  real, intent(in) :: a,b,c,deltaR,rho
  !f2py  optional  ,  depend(r)           ::  size1=len(r)
  !f2py  optional  ,  depend(position)    ::  size2=len(position[:,1,1])
  !f2py  optional  ,  depend(position)    ::  size3=len(position[1,:,1])
  !f2py  optional  ,  depend(indexAtom1)  ::  nba1=len(indexAtom1)
  !f2py  optional  ,  depend(indexAtom2)  ::  nba2=len(indexAtom2)
  real, intent(in)  ::  r(size1)
  real, intent(in)  ::  position(size2,size3,3)
  integer, intent(in)  ::  indexAtom1(nba1),indexAtom2(nba2)
  !output
  real(8),intent(out)  ::  gr(size1) 
  !local variable
  integer :: bin,j,k,l,nb
  real(8) :: distance
  real(8), dimension(3)  ::  p1=(/1,1,1/),p2=(/1,1,1/)
  real(8) :: HIST(size1)

  nb = 0
  j  = 1
  k  = 1
  l  = 1
  HIST(:) = 0


  do while(j<nbtime) !loop on step
    do k=1,size(indexAtom1)
      do l=1,size(indexAtom2)

        if (indexAtom1(k) /= indexAtom2(l)) then
          p1 = position(j,indexAtom1(k),:)
          p2 = position(j,indexAtom2(l),:)
        
          x = p1(1)-p2(1)
          y = p1(2)-p2(2)
          z = p1(3)-p2(3)
          
          x = min(abs(x),abs(x+a),abs(x-a))
          y = min(abs(y),abs(y+b),abs(y-b))
          z = min(abs(z),abs(z+c),abs(z-c))
          
          distance = (x**2+y**2+z**2)**0.5
          
          bin = int( distance / deltaR ) + 1
          
          if (bin <= maxbin) then
            
            HIST(bin) = HIST(bin) + 1
            
          end if
        end if
      end do
    end do
    j = j + inc
    nb = nb+1
  end do
  
  HIST =  HIST / (nb*(size(indexAtom1)*size(indexAtom2)))
  
  gr = HIST / (rho * ((4./3) * 3.14159265  * ((r + deltaR )**3 - (r**3) )))

end subroutine pair_distribution
