!{\src2tex{textfont=tt}}
!!****f* m_abi_linalg/abi_xcopy
!! NAME
!!  abi_xcopy
!!
!! FUNCTION
!!  abi_xcopy is the generic function for vector copy
!! It usually solve dst(1:n:incdst) = src(1:n:incsrc)
!! In many cases it assumes that data are contiguous in memory
!!
!! COPYRIGHT
!!  Copyright (C) 2001-2014 ABINIT group (LNguyen,FDahm (CS))
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~ABINIT/Infos/copyright
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

!!***

!!****f* m_abi_linalg/abi_zcopy
!! NAME
!! abi_zcopy
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zcopy(size,tsrc,incsrc,tdest,incdest,&
       timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zcopy'
!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    complex(dpc) :: tsrc
    complex(dpc) :: tdest
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Local variables-------------------------------
    real(dp) :: tsec(2)

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if

    call zcopy(size,tsrc,incsrc,tdest,incdest)

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if

  end subroutine abi_zcopy
!!***

!!****f* m_abi_linalg/abi_zcopy_1d
!! NAME
!! abi_zcopy_1d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zcopy_1d(size,tsrc,incsrc,tdest,incdest,&
       timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zcopy_1d'

!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    complex(dpc), dimension(:) :: tsrc
    complex(dpc), dimension(:) :: tdest
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim

    call abi_zcopy(size,tsrc(1),incsrc,tdest(1),incdest,&
         timopt,tim)

  end subroutine abi_zcopy_1d
!!***

!!****f* m_abi_linalg/abi_dcopy
!! NAME
!! abi_dcopy
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dcopy(size,tsrc,incsrc,tdest,incdest,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dcopy'
!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    real(dp),intent(in) :: tsrc
    real(dp),intent(out):: tdest
    !Optional Arguments----------------------------
    integer, intent(in), optional :: x_cplx,timopt,tim
    !Local variables-------------------------------
    real(dp) :: tsec(2)
    integer  :: cplx_

    !write(*,*) "dcopy size=",size 
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
    if(cplx_ == 2) then
       call zcopy(size,tsrc,incsrc,tdest,incdest)
    else
       call dcopy(size,tsrc,incsrc,tdest,incdest)
    end if

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if

  end subroutine abi_dcopy
!!***

!!****f* m_abi_linalg/abi_dcopy_1d
!! NAME
!! abi_dcopy_1d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dcopy_1d(size,tsrc,incsrc,tdest,incdest,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dcopy_1d'

!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    real(dp), dimension(:), intent(in)  :: tsrc
    real(dp), dimension(:), intent(out) :: tdest
    !Optional Arguments----------------------------
    integer, intent(in), optional :: timopt,tim
    !Only for lobpcgwf
    integer, intent(in), optional :: x_cplx
    
    call abi_dcopy(size,tsrc(1),incsrc,tdest(1),incdest,&
         x_cplx,timopt,tim)    
    
  end subroutine abi_dcopy_1d
!!***

!!****f* m_abi_linalg/abi_dcopy_2d
!! NAME
!! abi_dcopy_2d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dcopy_2d(size,tsrc,incsrc,tdest,incdest,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dcopy_2d'

!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    real(dp), dimension(:,:) :: tsrc
    real(dp), dimension(:,:) :: tdest
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Only for lobpcgwf
    integer, intent(in), optional :: x_cplx
    !Local variables-------------------------------

    ! write(*,*) "dcopy2D size=",size 
    call abi_dcopy(size,tsrc(1,1),incsrc,tdest(1,1),incdest,&
         x_cplx,timopt,tim)
    return
  end subroutine abi_dcopy_2d
!!***

!!****f* m_abi_linalg/abi_dcopy_0d_1d
!! NAME
!! abi_dcopy_0d_1d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dcopy_0d_1d(size,tsrc,incsrc,tdest,incdest,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dcopy_0d_1d'

!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    real(dp) :: tsrc
    real(dp), dimension(:) :: tdest
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !only lobpcgwf
    integer, optional :: x_cplx

    call abi_dcopy(size,tsrc,incsrc,tdest(1),incdest,&
       x_cplx,timopt,tim)

    return
  end subroutine abi_dcopy_0d_1d
!!***

!!****f* m_abi_linalg/abi_dcopy_1d_0d
!! NAME
!! abi_dcopy_1d_0d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dcopy_1d_0d(size,tsrc,incsrc,tdest,incdest,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dcopy_1d_0d'

!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    real(dp), dimension(:) :: tsrc
    real(dp) :: tdest
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !only lobpcgwf
    integer, optional :: x_cplx

    call abi_dcopy(size,tsrc(1),incsrc,tdest,incdest,&
         x_cplx,timopt,tim)

    return
  end subroutine abi_dcopy_1d_0d
!!***

!!****f* m_abi_linalg/abi_d2zcopy_2d
!! NAME
!! abi_d2zcopy_2d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_d2zcopy_2d(size,tsrc,incsrc,tdest,incdest,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_d2zcopy_2d'
!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    real(dp), dimension(:,:) :: tsrc
    complex(dpc), dimension(:,:) :: tdest
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !only in lobpcgwf
    integer, optional :: x_cplx
    !Local variables-------------------------------
    real(dp) :: tsec(2)
    integer  :: cplx_

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
    if(cplx_ == 2) then
       call zcopy(size,tsrc,incsrc,tdest,incdest)
    else
       call dcopy(size,tsrc,incsrc,tdest,incdest)
    end if
    
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if

  end subroutine abi_d2zcopy_2d
!!***

!!****f* m_abi_linalg/abi_z2dcopy_2d
!! NAME
!! abi_z2dcopy_2d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_z2dcopy_2d(size,tsrc,incsrc,tdest,incdest,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_z2dcopy_2d'
!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    integer, intent(in) :: size
    integer, intent(in) :: incsrc
    integer, intent(in) :: incdest
    complex(dpc), dimension(:,:) :: tsrc
    real(dp), dimension(:,:) :: tdest
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !only in lobpcgwf
    integer, optional :: x_cplx
    !Local variables-------------------------------
    real(dp) :: tsec(2)
    integer  :: cplx_

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
    if(cplx_ == 2) then
       call zcopy(size,tsrc,incsrc,tdest,incdest)
    else
       call dcopy(size,tsrc,incsrc,tdest,incdest)
    end if

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if

  end subroutine abi_z2dcopy_2d
!!***
