!{\src2tex{textfont=tt}}
!!****f* m_abi_linalg/abi_xgemm
!! NAME
!!  abi_xgemm
!!
!! FUNCTION
!!  abi_xgemm is the generic function that solve :
!! *     C := alpha*op( A )*op( B ) + beta*C,
!! *
!! *  where  op( X ) is one of
!! *
!! *     op( X ) = X   or   op( X ) = X**T,
!! *
!! *  alpha and beta are scalars, and A, B and C are matrices, with op( A )
!! *  an m by k matrix,  op( B )  a  k by n matrix and  C an m by n matrix.
!! In many cases it assumes that data are contiguous in memory
!!
!! COPYRIGHT
!!  Copyright (C) 2001-2014 ABINIT group (LNguyen,FDahm (CS))
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~ABINIT/Infos/copyright
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

!!***

!!****f* m_abi_linalg/abi_zgemm
!! NAME
!! abi_zgemm
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zgemm(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,&
       timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zgemm'
!End of the abilint section

    implicit none
    !Arguments------------------------------------
    integer :: K
    integer :: LDA
    integer :: LDB
    integer :: LDC
    integer :: M
    integer :: N
    complex(dpc) :: A
    complex(dpc) :: ALPHA
    complex(dpc) :: B
    complex(dpc) :: BETA
    complex(dpc) :: C
    character(len=1) :: TRANSA
    character(len=1) :: TRANSB
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Local variables-------------------------------
    real(dp) :: tsec(2)

#if defined HAVE_LINALG_PLASMA
    integer :: trans_plasma_a, trans_plasma_b
    logical :: LSAME
    external LSAME
    integer :: abi_info
    include 'plasmaf.h'
#endif
    
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3)then
        call timab(tim,1,tsec)
      end if
    end if

#ifdef HAVE_LINALG_PLASMA
    if(LSAME(TRANSA,'C')) then
       trans_plasma_a = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_a = PlasmaTrans
    else
       trans_plasma_a = PlasmaNoTrans
    end if
    if(LSAME(TRANSB,'C')) then
       trans_plasma_b = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_b = PlasmaTrans
    else
       trans_plasma_b = PlasmaNoTrans
    end if
    call PLASMA_zgemm(trans_plasma_a,trans_plasma_b,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,abi_info)

#else
    
    call zgemm(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC)

#endif
    
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3)then
        call timab(tim,2,tsec)
      end if
    end if

    return
  end subroutine abi_zgemm
!!***

!!****f* m_abi_linalg/abi_zgemm_1d
!! NAME
!! abi_zgemm_1d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zgemm_1d(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,&
       timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zgemm_1d'

!End of the abilint section

    implicit none
    !Arguments------------------------------------
    integer :: K
    integer :: LDA
    integer :: LDB
    integer :: LDC
    integer :: M
    integer :: N
    complex(dpc), dimension(:),intent(in) :: A
    complex(dpc) :: ALPHA
    complex(dpc), dimension(:),intent(in) :: B
    complex(dpc) :: BETA
    complex(dpc), dimension(:),intent(inout) :: C
    character(len=1) :: TRANSA
    character(len=1) :: TRANSB
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim

    call abi_zgemm(TRANSA,TRANSB,M,N,K,ALPHA,A(1),LDA,B(1),LDB,BETA,C(1),LDC,&
         timopt,tim)
    return
  end subroutine abi_zgemm_1d
!!***

!!****f* m_abi_linalg/abi_zgemm_2d
!! NAME
!! abi_zgemm_2d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zgemm_2d(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,&
       timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zgemm_2d'

!End of the abilint section

    implicit none
    !Arguments------------------------------------
    integer :: K
    integer :: LDA
    integer :: LDB
    integer :: LDC
    integer :: M
    integer :: N
    complex(dpc), dimension(:,:),intent(in) :: A
    complex(dpc) :: ALPHA
    complex(dpc), dimension(:,:),intent(in) :: B
    complex(dpc) :: BETA
    complex(dpc), dimension(:,:),intent(inout) :: C
    character(len=1) :: TRANSA
    character(len=1) :: TRANSB
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    call abi_zgemm(TRANSA,TRANSB,M,N,K,ALPHA,A(1,1),LDA,B(1,1),LDB,BETA,C(1,1),LDC,&
         timopt,tim)
    return
  end subroutine abi_zgemm_2d
!!***

!!****f* m_abi_linalg/abi_dgemm
!! NAME
!! abi_dgemm
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dgemm(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dgemm'
!End of the abilint section

    implicit none
    !Arguments------------------------------------
    integer :: K
    integer :: LDA
    integer :: LDB
    integer :: LDC
    integer :: M
    integer :: N
    real(dp), dimension(:,:) :: A
    real(dp) :: ALPHA
    real(dp), dimension(:,:) :: B
    real(dp) :: BETA
    real(dp), dimension(:,:) :: C
    character(len=1) :: TRANSA
    character(len=1) :: TRANSB
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Only for lobpcgwf
    integer, intent(in), optional :: x_cplx
    !Local variables-------------------------------
    real(dp) :: tsec(2)
    integer  :: cplx_
#if defined HAVE_LINALG_PLASMA
    integer :: trans_plasma_a, trans_plasma_b
    logical :: LSAME
    external LSAME
    integer :: abi_info
    include 'plasmaf.h'
#endif
    
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3)then
        call timab(tim,1,tsec)
      end if
    end if
    
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
    
#ifdef HAVE_LINALG_PLASMA
!    write(*,* ) " abi_dgemm => plasma_[zd]gemm"
    if(LSAME(TRANSA,'C')) then
       trans_plasma_a = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_a = PlasmaTrans
    else
       trans_plasma_a = PlasmaNoTrans
    end if
    if(LSAME(TRANSB,'C')) then
       trans_plasma_b = PlasmaConjTrans
    else if(LSAME(TRANSB,'T')) then
       trans_plasma_b = PlasmaTrans
    else
       trans_plasma_b = PlasmaNoTrans
    end if
    if(present(x_cplx) .and. (x_cplx == 2)) then
       call PLASMA_zgemm(trans_plasma_a,trans_plasma_b,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,abi_info)
    else
       call PLASMA_dgemm(trans_plasma_a,trans_plasma_b,M,N,K,real(ALPHA,dp),A,LDA,B,LDB,real(BETA,dp),C,LDC,abi_info)
    end if

#else

    if(cplx_ == 2)then
       call zgemm(TRANSA,TRANSB,M,N,K,cmplx(ALPHA,0.d0,dpc),A,LDA,B,LDB,cmplx(BETA,0.d0,dpc),C,LDC)
    else
       call dgemm(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
    end if
    
#endif

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3)then
        call timab(tim,2,tsec)
      end if
    end if
    
  end subroutine abi_dgemm
!!***

!!****f* m_abi_linalg/abi_d2zgemm
!! NAME
!! abi_d2zgemm
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_d2zgemm(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_d2zgemm'
!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    integer, intent(in) :: lda
    integer, intent(in) :: ldb
    integer, intent(in) :: ldc
    integer, intent(in) :: m
    integer, intent(in) :: n
    integer, intent(in) :: k
    real(dp), dimension(:,:), intent(in) :: a
    real(dp), dimension(:,:), intent(in) :: b
    real(dp), dimension(:,:), intent(inout) :: c
    complex(dpc), intent(in) :: alpha
    complex(dpc), intent(in) :: beta
    character(len=1), intent(in) :: transa
    character(len=1), intent(in) :: transb
    !Optional Arguments----------------------------
    integer, intent(in), optional :: timopt,tim
    !Only for lobpcgwf
    integer, intent(in), optional :: x_cplx

    !Local variables-------------------------------
    real(dp) :: tsec(2)
    integer  :: cplx_
#if defined HAVE_LINALG_PLASMA
    integer :: trans_plasma_a, trans_plasma_b
    logical :: LSAME
    external LSAME
    integer :: abi_info
    include 'plasmaf.h'
#endif
    
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3)then
        call timab(tim,1,tsec)
      end if
    end if
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
#if defined HAVE_LINALG_PLASMA
!    write(*,* ) " abi_d2zgemm => plasma_[zd]gemm"
    if(LSAME(TRANSA,'C')) then
       trans_plasma_a = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_a = PlasmaTrans
    else
       trans_plasma_a = PlasmaNoTrans
    end if
    if(LSAME(TRANSB,'C')) then
       trans_plasma_b = PlasmaConjTrans
    else if(LSAME(TRANSB,'T')) then
       trans_plasma_b = PlasmaTrans
    else
       trans_plasma_b = PlasmaNoTrans
    end if
    if(cplx_ == 2) then
       call PLASMA_zgemm(trans_plasma_a,trans_plasma_b,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC,abi_info)
    else
       call PLASMA_dgemm(trans_plasma_a,trans_plasma_b,M,N,K,real(ALPHA,dp),A,LDA,B,LDB,real(BETA,dp),C,LDC,abi_info)
    end if

#else
    
    if(cplx_ == 2) then
       call zgemm(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
    else
       call dgemm(TRANSA,TRANSB,M,N,K,real(ALPHA,dp),A,LDA,B,LDB,real(BETA,dp),C,LDC)
    end if

#endif

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3)then
        call timab(tim,2,tsec)
      end if
    end if
    
  end subroutine abi_d2zgemm
!!***
