!{\src2tex{textfont=tt}}
!!****f* m_abi_linalg/abi_xheev
!! NAME
!!  abi_xheev
!!
!! FUNCTION
!!  abi_xheev is the generic function that compute 
!! all eigenvalues and, optionally, eigenvectors of a
!! symmetric or hermitian matrix A.
!!
!! COPYRIGHT
!!  Copyright (C) 2001-2014 ABINIT group (LNguyen,FDahm (CS))
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~ABINIT/Infos/copyright
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

!!***

!!****f* m_abi_linalg/abi_dheev
!! NAME
!! abi_dheev
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dheev(jobz,uplo,n,a,lda,w,work,lwork,rwork,info, &
       x_cplx,istwf_k,timopt,tim_xeigen,use_slk,use_gpu)
    use defs_basis
#if defined HAVE_LINALG_PLASMA
    use iso_c_binding
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dheev'
!End of the abilint section

    implicit none

    !Arguments ------------------------------------
    character(len=1), intent(in) :: jobz
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    real(dp), intent(inout) :: a(:,:)
    integer, intent(in) :: lda
    real(dp), intent(inout) :: work(:)
    integer, intent(in) :: lwork
    real(dp),optional,intent(inout) :: rwork(lwork)
    real(dp), intent(out) :: w(n)
    integer, intent(out) :: info
    !Optional Arguments ------------------------------------
    integer, optional, intent(in) :: x_cplx
    integer, optional, intent(in) :: timopt,tim_xeigen,use_gpu
    integer, optional, intent(in) :: use_slk
    integer, optional, intent(in) :: istwf_k
    !Local variables-------------------------------
    character(len=500) :: msg
    integer :: istwf_k_ , usegpu_
    real(dp) :: tsec(2)
    integer  :: cplx_
#if defined HAVE_LINALG_MAGMA
    integer :: liwork,lwork_, lrwork_
    integer, dimension(:), allocatable :: iwork
#endif
#if defined HAVE_LINALG_PLASMA
    logical,external :: LSAME
    integer :: jobz_plasma_a
    integer :: uplo_plasma_a
    type(c_ptr) :: plasma_work
    include 'plasmaf.h'
#endif

    ! *********************************************************************

    if (present(tim_xeigen).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim_xeigen,1,tsec)
      end if
    end if
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
    usegpu_=0;if (present(use_gpu)) usegpu_=use_gpu
    istwf_k_=1;if (present(istwf_k)) istwf_k_=istwf_k
   
    if( n > eigen_d_maxsize ) then
       write(msg,'(a,2i3)')' Eigen size higher than max size set!!',n,eigen_d_maxsize
       MSG_ERROR(msg)
    endif

    if(present(use_slk)) then
       ABI_UNUSED(use_slk)
    end if
    
#if defined HAVE_LINALG_MAGMA
    if (usegpu_==1) then
       !work only if  lwork=n**2 + 33*n?
       lrwork_= 1 + 5*n + 2*(n**2)
       liwork = 3 + 5*n
       ABI_ALLOCATE(iwork,(liwork))
       if (cplx_ == 2 .and. present(rwork)) then
          lwork_=33*n + (n**2)
          call magmaf_zheevd(jobz,uplo,n,a,lda,w,work(1:2*lwork_),lwork_,rwork(1:lrwork_),lrwork_,iwork,liwork,info)
       else
          lwork_= 1+n*(6 + 2*n)
          call magmaf_dsyevd(jobz,uplo,n,a,lda,w,work(1:lwork_),lwork_,iwork(1:liwork),liwork,info)
       endif
       ABI_DEALLOCATE(iwork)
    else
#endif

#if defined HAVE_LINALG_SCALAPACK
       if( (present(use_slk) .and. use_slk == 1).and.( n > maxval(abi_processor%grid%dims(1:2))) )  then
          call compute_eigen1(abi_communicator,abi_processor,n,n,a,w,istwf_k_)
          info = 0 ! This is to avoid unwanted warning but it's not clean
       else
#endif

#if defined HAVE_LINALG_PLASMA   
          ! FDahm & LNGuyen  (November 2012) : 
          !  In Plasma v 2.4.6, eigen routines support only 
          !the eigenvalues computation (jobz=N) and not the
          !full eigenvectors bases determination (jobz=V)
          if(LSAME(jobz,'N')) then
             jobz_plasma_a = PlasmaNoVec
             ! if(LSAME(jobz,'N')) then
             !    jobz_plasma_a = PlasmaNoVec
             ! else 
             !    jobz_plasma_a = PlasmaVec
             ! end if
             if(LSAME(uplo,'U')) then
                uplo_plasma_a = PlasmaUpper
             else
                uplo_plasma_a = PlasmaLower
             end if
             if ( cplx_ == 2 .and. present(rwork)) then
                call PLASMA_Alloc_Workspace_zheev(n,n,plasma_work,info)
                call PLASMA_zheev(jobz_plasma_a,uplo_plasma_a,n,a,lda,w,plasma_work,rwork,lwork,info)
                call PLASMA_Dealloc_handle(plasma_work,info)
             else
                call PLASMA_Alloc_Workspace_dsyev(n,n,plasma_work,info)
                call PLASMA_dsyev(jobz_plasma_a,uplo_plasma_a,n,a,lda,w,plasma_work,rwork,lwork,info)
                call PLASMA_Dealloc_handle(plasma_work,info)
             endif
          else
#endif
             if ( cplx_ == 2 .and. present(rwork)) then
                call zheev(jobz,uplo,n,a,lda,w,work,lwork/2,rwork,info)
             else
                call dsyev(jobz,uplo,n,a,lda,w,work,lwork,info)
             endif
#if defined HAVE_LINALG_PLASMA
          end if
#endif
#if defined HAVE_LINALG_SCALAPACK
       end if
#endif
#if defined HAVE_LINALG_MAGMA
    end if
#endif
    
    if(info/=0) then
       write(msg,'(a,i3)')' Problem in abi_xheev, info= ',info
       MSG_WARNING(msg)
    endif
    
    if (present(tim_xeigen).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim_xeigen,2,tsec)
      end if
    end if
    return
  end subroutine abi_dheev
!!***

!!****f* m_abi_linalg/abi_dheev_alloc
!! NAME
!! abi_dheev_alloc
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dheev_alloc(jobz,uplo,n,a,w, &
       x_cplx,istwf_k,timopt,tim_xeigen,use_slk,use_gpu)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dheev_alloc'

!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    character(len=1), intent(in) :: jobz
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    real(dp), intent(inout) :: a(:,:)
    real(dp), intent(out) :: w(n) 
    integer :: info
    !Optional Arguments ------------------------------------
    integer, optional, intent(in) :: x_cplx
    integer, optional, intent(in) :: timopt,tim_xeigen,use_slk,use_gpu
    integer, optional, intent(in) :: istwf_k
    
    ! *********************************************************************
    
    call abi_dheev(jobz,uplo,n,a,n,w,eigen_d_work,eigen_d_lwork,eigen_z_rwork,info, &
         x_cplx,istwf_k,timopt,tim_xeigen,use_slk,use_gpu)
    return
  end subroutine abi_dheev_alloc
!!***

!!****f* m_abi_linalg/abi_cheev
!! NAME
!! abi_cheev
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_cheev(jobz,uplo,n,a,lda,w,work,lwork,rwork,info)
    use defs_basis
#if defined HAVE_LINALG_PLASMA
    !use plasma, plasma_dp => dp
    use iso_c_binding
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_cheev'
!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    character(len=1), intent(in) :: jobz
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    complex(spc), intent(inout) :: a(:,:)
    integer, intent(in) :: lda
    complex(spc), intent(inout) :: work(:)
    integer, intent(in) :: lwork
    real(sp), intent(inout) :: rwork(lwork)
    real(sp), intent(out) :: w(n)
    integer, intent(out) :: info
#if defined HAVE_LINALG_PLASMA
    !Optional Arguments ------------------------------------
    logical,external :: LSAME
    integer :: jobz_plasma_a
    integer :: uplo_plasma_a
    type(c_ptr) :: plasma_work
    include 'plasmaf.h'
#endif
    ! *********************************************************************
    
#if defined HAVE_LINALG_PLASMA
    ! FDahm & LNGuyen  (November 2012) : 
    !  In Plasma v 2.4.6, eigen routines support only 
    !the eigenvalues computation (jobz=N) and not the
    !full eigenvectors bases determination (jobz=V)
    if(LSAME(jobz,'N')) then
       jobz_plasma_a = PlasmaNoVec
       ! if(LSAME(jobz,'N')) then
       !    jobz_plasma_a = PlasmaNoVec
       ! else 
       !    jobz_plasma_a = PlasmaVec
       ! end if
       if(LSAME(uplo,'U')) then
          uplo_plasma_a = PlasmaUpper
       else
          uplo_plasma_a = PlasmaLower
       end if
       call PLASMA_Alloc_Workspace_cheev(n,n,plasma_work,info)
       call PLASMA_cheev(jobz_plasma_a,uplo_plasma_a,n,a,lda,w,plasma_work,rwork,lwork,info)
       call PLASMA_Dealloc_handle(plasma_work,info)
    else
#endif
       call cheev(jobz,uplo,n,a,lda,w,work,lwork,rwork,info)
#if defined HAVE_LINALG_PLASMA
    end if
#endif
    return
  end subroutine abi_cheev
!!***

!!****f* m_abi_linalg/abi_cheev_alloc
!! NAME
!! abi_cheev_alloc
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_cheev_alloc(jobz,uplo,n,a,w)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_cheev_alloc'

!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    character(len=1), intent(in) :: jobz
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    complex(spc), intent(inout) :: a(:,:)
    real(sp), intent(out) :: w(n)
    integer :: info

    ! *********************************************************************
    
    call abi_cheev(jobz,uplo,n,a,n,w,eigen_c_work,eigen_c_lwork,eigen_c_rwork,info)
    return
  end subroutine abi_cheev_alloc
!!***

!!****f* m_abi_linalg/abi_zheev
!! NAME
!! abi_zheev
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zheev(jobz,uplo,n,a,lda,w,work,lwork,rwork,info)
    use defs_basis
#if defined HAVE_LINALG_PLASMA
    !use plasma, plasma_dp => dp
    use iso_c_binding
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zheev'
!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    character(len=1), intent(in) :: jobz
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    complex(dpc), intent(inout) :: a(:,:)
    integer, intent(in) :: lda
    complex(dpc), intent(inout) :: work(:)
    integer, intent(in) :: lwork
    real(dp), intent(inout) :: rwork(lwork)
    real(dp), intent(out) :: w(n)
    integer, intent(out) :: info
#if defined HAVE_LINALG_PLASMA
    !Optional Arguments ------------------------------------
    logical,external :: LSAME
    integer :: jobz_plasma_a
    integer :: uplo_plasma_a
    type(c_ptr) :: plasma_work
    include 'plasmaf.h'
#endif
    ! *********************************************************************
    
#if defined HAVE_LINALG_PLASMA
    ! FDahm & LNGuyen  (November 2012) : 
    !  In Plasma v 2.4.6, eigen routines support only 
    !the eigenvalues computation (jobz=N) and not the
    !full eigenvectors bases determination (jobz=V)
    if(LSAME(jobz,'N')) then
       jobz_plasma_a = PlasmaNoVec
       ! if(LSAME(jobz,'N')) then
       !    jobz_plasma_a = PlasmaNoVec
       ! else 
       !    jobz_plasma_a = PlasmaVec
       ! end if
       if(LSAME(uplo,'U')) then
          uplo_plasma_a = PlasmaUpper
       else
          uplo_plasma_a = PlasmaLower
       end if
       call PLASMA_Alloc_Workspace_zheev(n,n,plasma_work,info)
       call PLASMA_zheev(jobz_plasma_a,uplo_plasma_a,n,a,lda,w,plasma_work,rwork,lwork,info)
       call PLASMA_Dealloc_handle(plasma_work,info)
    else
#endif
       call zheev(jobz,uplo,n,a,lda,w,work,lwork,rwork,info)
#if defined HAVE_LINALG_PLASMA
    end if
#endif
    return
  end subroutine abi_zheev
!!***

!!****f* m_abi_linalg/abi_zheev_alloc
!! NAME
!! abi_zheev_alloc
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_zheev_alloc(jobz,uplo,n,a,w)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_zheev_alloc'

!End of the abilint section

    implicit none
    !Arguments ------------------------------------
    character(len=1), intent(in) :: jobz
    character(len=1), intent(in) :: uplo
    integer, intent(in) :: n
    complex(dpc), intent(inout) :: a(:,:)
    real(dp), intent(out) :: w(n)
    integer :: info

    ! *********************************************************************
    
    call abi_zheev(jobz,uplo,n,a,n,w,eigen_z_work,eigen_z_lwork,eigen_z_rwork,info)
    return
  end subroutine abi_zheev_alloc
!!***
