!{\src2tex{textfont=tt}}
!!****f* m_abi_linalg/abi_xtrsm
!! NAME
!!  abi_xtrsm
!!
!! FUNCTION
!!  abi_xtrsm is the generic function that solve :
!! *     op( A )*X = alpha*B,   or   X*op( A ) = alpha*B,
!! *
!! *  where alpha is a scalar, X and B are m by n matrices, A is a unit, or
!! *  non-unit,  upper or lower triangular matrix  and  op( A )  is one  of
!! *
!! *     op( A ) = A   or   op( A ) = A**T.
!! *
!! *  The matrix X is overwritten on B.
!! In many cases it assumes that data are contiguous in memory
!!
!! COPYRIGHT
!!  Copyright (C) 2001-2014 ABINIT group (LNguyen,FDahm (CS))
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~ABINIT/Infos/copyright
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

!!***

!!****f* m_abi_linalg/abi_ztrsm
!! NAME
!! abi_ztrsm
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_ztrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb,&
       timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_ztrsm'
!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    character(len=1), intent(in) :: side
    character(len=1), intent(in) :: uplo
    character(len=1), intent(in) :: transa
    character(len=1), intent(in) :: diag
    integer, intent(in) :: m
    integer, intent(in) :: n
    complex(dpc), intent(in) :: alpha
    complex(dpc), dimension(:,:), intent(in) :: a
    integer, intent(in) :: lda
    complex(dpc), dimension(:,:), intent(inout) :: b
    integer, intent(in) :: ldb
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Local variables-------------------------------
    real(dp) :: tsec(2)
#if defined HAVE_LINALG_PLASMA
    integer :: abi_info
    integer :: trans_plasma_a
    integer :: side_plasma_a
    integer :: uplo_plasma_a
    integer :: diag_plasma_a
    logical :: LSAME
    external LSAME
    include 'plasmaf.h'
#endif

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if

#if defined HAVE_LINALG_PLASMA
    if(LSAME(TRANSA,'C')) then
       trans_plasma_a = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_a = PlasmaTrans
    else
       trans_plasma_a = PlasmaNoTrans
    end if

    if(LSAME(side,'L')) then
       side_plasma_a = PlasmaLeft
    else
       side_plasma_a = PlasmaRight
    end if

    if(LSAME(uplo,'U')) then
       uplo_plasma_a = PlasmaUpper
    else
       uplo_plasma_a = PlasmaLower
    end if

    if(LSAME(diag,'U')) then
       diag_plasma_a = PlasmaUnit
    else
       diag_plasma_a = PlasmaNonUnit
    end if

    call PLASMA_ztrsm(side_plasma_a,uplo_plasma_a,trans_plasma_a,diag_plasma_a,m,n,alpha,a,lda,b,ldb,abi_info)

#else
    
    call ztrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb)

#endif
    
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if

  end subroutine abi_ztrsm
!!***

!!****f* m_abi_linalg/abi_dtrsm
!! NAME
!! abi_dtrsm
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_dtrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_dtrsm'
!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    character(len=1), intent(in) :: side
    character(len=1), intent(in) :: uplo
    character(len=1), intent(in) :: transa
    character(len=1), intent(in) :: diag
    integer, intent(in) :: m
    integer, intent(in) :: n
    real(dp), intent(in) :: alpha
    real(dp), dimension(:,:), intent(in) :: a
    integer, intent(in) :: lda
    real(dp), dimension(:,:), intent(inout) :: b
    integer, intent(in) :: ldb
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Only for lobpcgwf
    integer, intent(in), optional :: x_cplx
    !Local variables-------------------------------
    integer  :: cplx_
    real(dp) :: tsec(2)
#if defined HAVE_LINALG_PLASMA
    integer :: abi_info
    integer :: trans_plasma_a
    integer :: side_plasma_a
    integer :: uplo_plasma_a
    integer :: diag_plasma_a
    logical :: LSAME
    external LSAME
    include 'plasmaf.h'
#endif

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx
    
#if defined HAVE_LINALG_PLASMA
    if(LSAME(TRANSA,'C')) then
       trans_plasma_a = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_a = PlasmaTrans
    else
       trans_plasma_a = PlasmaNoTrans
    end if

    if(LSAME(side,'L')) then
       side_plasma_a = PlasmaLeft
    else
       side_plasma_a = PlasmaRight
    end if

    if(LSAME(uplo,'U')) then
       uplo_plasma_a = PlasmaUpper
    else
       uplo_plasma_a = PlasmaLower
    end if

    if(LSAME(diag,'U')) then
       diag_plasma_a = PlasmaUnit
    else
       diag_plasma_a = PlasmaNonUnit
    end if
      
    if(cplx_ == 2) then
       call PLASMA_ztrsm(side_plasma_a,uplo_plasma_a,trans_plasma_a,diag_plasma_a,m,n,cmplx(alpha,0.d0,dpc),a,lda,b,ldb,abi_info)
    else
       call PLASMA_dtrsm(side_plasma_a,uplo_plasma_a,trans_plasma_a,diag_plasma_a,m,n,alpha,a,lda,b,ldb,abi_info)
    end if
    
#else    
  
    if(cplx_ == 2) then
       call ztrsm(side,uplo,transa,diag,m,n,cmplx(alpha,0.d0,dpc),a,lda,b,ldb)
    else
       call dtrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb)
    end if

#endif

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if

    return
  end subroutine abi_dtrsm
!!***

!!****f* m_abi_linalg/abi_d2ztrsm
!! NAME
!! abi_d2ztrsm
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_d2ztrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb,&
       x_cplx,timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_d2ztrsm'
!End of the abilint section

    implicit none    
    !Arguments-------------------------------------
    character(len=1), intent(in) :: side
    character(len=1), intent(in) :: uplo
    character(len=1), intent(in) :: transa
    character(len=1), intent(in) :: diag
    integer, intent(in) :: m
    integer, intent(in) :: n
    complex(dpc), intent(in) :: alpha
    real(dp), dimension(:,:), intent(in) :: a
    integer, intent(in) :: lda
    real(dp), dimension(:,:), intent(inout) :: b
    integer, intent(in) :: ldb
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Only for lobpcgwf
    integer, intent(in), optional :: x_cplx
    !Local variables-------------------------------
    real(dp) :: tsec(2)
    integer  :: cplx_
#if defined HAVE_LINALG_PLASMA
    integer :: abi_info
    integer :: trans_plasma_a
    integer :: side_plasma_a
    integer :: uplo_plasma_a
    integer :: diag_plasma_a
    logical :: LSAME
    external LSAME
    include 'plasmaf.h'
#endif

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if
    cplx_=1 ; if(PRESENT(x_cplx)) cplx_ = x_cplx

#if defined HAVE_LINALG_PLASMA
    if(LSAME(TRANSA,'C')) then
       trans_plasma_a = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_a = PlasmaTrans
    else
       trans_plasma_a = PlasmaNoTrans
    end if

    if(LSAME(side,'L')) then
       side_plasma_a = PlasmaLeft
    else
       side_plasma_a = PlasmaRight
    end if

    if(LSAME(uplo,'U')) then
       uplo_plasma_a = PlasmaUpper
    else
       uplo_plasma_a = PlasmaLower
    end if

    if(LSAME(diag,'U')) then
       diag_plasma_a = PlasmaUnit
    else
       diag_plasma_a = PlasmaNonUnit
    end if
      
    if(cplx_ == 2) then
       call PLASMA_ztrsm(side_plasma_a,uplo_plasma_a,trans_plasma_a,diag_plasma_a,m,n,alpha,a,lda,b,ldb,abi_info)
    else
       call PLASMA_dtrsm(side_plasma_a,uplo_plasma_a,trans_plasma_a,diag_plasma_a,m,n,real(alpha,dp),a,lda,b,ldb,abi_info)
    end if

#else    
    
    if(cplx_ == 2) then
       call ztrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb)
    else
       call dtrsm(side,uplo,transa,diag,m,n,real(alpha,dp),a,lda,b,ldb)
    end if

#endif
    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if

    return
  end subroutine abi_d2ztrsm
!!***

!!****f* m_abi_linalg/abi_d2ztrsm_3d
!! NAME
!! abi_d2ztrsm_3d
!!
!! FUNCTION
!! 
!!
!! COPYRIGHT
!!   Copyright (C) 1998-2014 ABINIT group (LNguyen, FDahm (CS) )
!!   this file is distributed under the terms of the
!!   gnu general public license, see ~abinit/COPYING
!!   or http://www.gnu.org/copyleft/gpl.txt .
!!   for the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   
!!
!! PARENTS
!!      
!!
!! CHILDREN
!!
!! SOURCE
!!
  subroutine abi_d2ztrsm_3d(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb,&
       timopt,tim)
    use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'abi_d2ztrsm_3d'
!End of the abilint section

    implicit none
    !Arguments-------------------------------------
    character(len=1), intent(in) :: side
    character(len=1), intent(in) :: uplo
    character(len=1), intent(in) :: transa
    character(len=1), intent(in) :: diag
    integer, intent(in) :: m
    integer, intent(in) :: n
    complex(dpc), intent(in) :: alpha
    real(dp), dimension(:,:,:), intent(in) :: a
    integer, intent(in) :: lda
    real(dp), dimension(:,:,:), intent(inout) :: b
    integer, intent(in) :: ldb
    !Optional Arguments----------------------------
    integer, optional :: timopt,tim
    !Local variables-------------------------------
    real(dp) :: tsec(2)
#if defined HAVE_LINALG_PLASMA
    integer :: abi_info
    integer :: trans_plasma_a
    integer :: side_plasma_a
    integer :: uplo_plasma_a
    integer :: diag_plasma_a
    logical :: LSAME
    external LSAME
    include 'plasmaf.h'
#endif

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,1,tsec)
      end if
    end if

#if defined HAVE_LINALG_PLASMA
    if(LSAME(TRANSA,'C')) then
       trans_plasma_a = PlasmaConjTrans
    else if(LSAME(TRANSA,'T')) then
       trans_plasma_a = PlasmaTrans
    else
       trans_plasma_a = PlasmaNoTrans
    end if

    if(LSAME(side,'L')) then
       side_plasma_a = PlasmaLeft
    else
       side_plasma_a = PlasmaRight
    end if

    if(LSAME(uplo,'U')) then
       uplo_plasma_a = PlasmaUpper
    else
       uplo_plasma_a = PlasmaLower
    end if

    if(LSAME(diag,'U')) then
       diag_plasma_a = PlasmaUnit
    else
       diag_plasma_a = PlasmaNonUnit
    end if

    call PLASMA_ztrsm(side_plasma_a,uplo_plasma_a,trans_plasma_a,diag_plasma_a,m,n,alpha,a,lda,b,ldb)

#else
    
    call ztrsm(side,uplo,transa,diag,m,n,alpha,a,lda,b,ldb)

#endif    

    if (present(tim).and.present(timopt)) then
      if(abs(timopt)==3) then
        call timab(tim,2,tsec)
      end if
    end if
    return
  end subroutine abi_d2ztrsm_3d
!!***
