!{\src2tex{textfont=tt}}
!!****f* ABINIT/qmc_prep_ctqmc
!! NAME
!! qmc_prep_ctqmc
!!
!! FUNCTION
!! Prepare and call the qmc subroutines
!!
!! COPYRIGHT
!! Copyright (C) 1999-2011 ABINIT group (BAmadon)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  cryst_struc <type(crystal_structure)>=crystal structure data
!!  hu <type(hu_type)>= U interaction
!!  paw_dmft <type(paw_dmft_type)>= DMFT data structure
!!  pawang <type(pawang)>=paw angular mesh and related data
!!  pawprtvol = drive the amount of writed data.
!!  weiss <type(green_type)>= weiss function
!!
!! OUTPUT
!!  green <type(green_type)>= green function
!!
!! NOTES
!!
!! PARENTS
!!      impurity_solve
!!
!! CHILDREN
!!      bathoperator_printf,compute_levels,copy_green,copy_matlu
!!      ctqmc_printgreen,ctqmcinterface_run,destroy_green,destroy_matlu
!!      destroy_oper,destroy_self,diag_matlu,diff_matlu,dyson,flush_unit
!!      init_green,init_matlu,init_oper,initialize_self,inverse_oper,leave_new
!!      nullify_matlu,occup_green_tau,print_green,print_matlu,printocc_green
!!      rotate_matlu,spline_fct,sym_matlu,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine qmc_prep_ctqmc(cryst_struc,green,hdc,hu,paw_dmft,pawang,pawprtvol,weiss)

 use defs_basis
 use defs_datatypes
 use defs_abitypes
 use m_errors
 use m_pawang, only : pawang_type
 use m_crystal, only : crystal_structure
 use m_green, only : green_type,occup_green_tau,print_green,printocc_green,spline_fct,copy_green,init_green,destroy_green
 use m_paw_dmft, only : paw_dmft_type
 use m_oper, only : oper_type,destroy_oper,init_oper,inverse_oper
 use m_matlu, only : matlu_type,sym_matlu, print_matlu, &
& diag_matlu,init_matlu,destroy_matlu,nullify_matlu,rotate_matlu,checkdiag_matlu, &
& copy_matlu, diff_matlu
 use m_hu, only : hu_type,rotatevee_hu
 use m_xmpi
 use m_ctqmc
 use m_CtqmcInterface
 use m_greenhyb
 use m_self, only : self_type,initialize_self,destroy_self,print_self,rw_self
 use m_io_tools, only : flush_unit

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'qmc_prep_ctqmc'
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_68_dmft, except_this_one => qmc_prep_ctqmc
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
! type(pawang_type), intent(in) :: pawang
 type(crystal_structure),intent(in) :: cryst_struc
 type(green_type), intent(out) :: green
 type(hu_type), intent(in) :: hu(cryst_struc%ntypat)
 type(paw_dmft_type), intent(inout)  :: paw_dmft
 type(pawang_type), intent(in) :: pawang
 integer, intent(in) :: pawprtvol
 type(green_type), intent(inout) :: weiss
 type(oper_type), intent(in) :: hdc

!Local variables ------------------------------
 character(len=500) :: message
 character(len=2) :: gtau_iter,iatomnb
 integer :: iatom,ierr,iflavor,ifreq,im,ispinor,isppol,itau,itypat,im2,ispinor2
 integer :: lpawu,master,mbandc,natom,nflavor,nkpt,nspinor,nsppol,nsppol_imp,tndim
 integer :: nproc,myproc,spacecomm,opt_diag
 complex(dpc) :: omega_current
 real(dp) :: Doccsum,Noise,nsweep
 real(dp) :: retemp
 real(dp) :: tsec(2)
! arrays
 real(dp), allocatable :: gtmp(:,:), levels_ctqmc(:)
 complex(dpc), allocatable :: fw1(:),f_with_k(:,:),gw_tmp(:,:)
 !type(coeff2_type), allocatable :: udens_atoms(:)
! Type    -----------------------------------------
 type(coeff2c_type), allocatable :: eigvectmatlu(:,:)
 type(green_type) :: weiss_for_rot
 type(matlu_type), allocatable :: dmat_diag(:)
 type(matlu_type), allocatable :: matlu1(:)
 type(matlu_type), allocatable :: level_diag(:)
 type(oper_type)  :: energy_level
 type(self_type) :: self
 type(green_type) :: gw_loc
 type(CtqmcInterface) :: hybrid
! ************************************************************************
 mbandc=paw_dmft%mbandc
 nkpt=paw_dmft%nkpt
 nsppol=paw_dmft%nsppol
 natom=paw_dmft%natom
 nspinor=paw_dmft%nspinor
 nsweep=paw_dmft%dmftqmc_n
 call init_green(weiss_for_rot,paw_dmft)
 call init_green(gw_loc,paw_dmft)
 call copy_green(weiss,weiss_for_rot,opt_tw=2)

!=======================================================================
!== Use one QMC solver   ===============================================
!=======================================================================
 write(message,'(2a)') ch10,'  ===  CT-QMC solver === '
 call wrtout(std_out,message,'COLL')

! Initialise for compiler
 omega_current=czero
 if(pawprtvol>0) then
 end if

! Initialise spaceComm, myproc, and nproc
 spacecomm=paw_dmft%spacecomm
 myproc=paw_dmft%myproc
 nproc=paw_dmft%nproc

! ======================================
! Allocations: diagonalization and eigenvectors
! ======================================
 ABI_DATATYPE_ALLOCATE(level_diag,(natom))
 !ABI_DATATYPE_ALLOCATE(udens_atoms,(natom))
 ABI_DATATYPE_ALLOCATE(eigvectmatlu,(natom,nsppol))
 ABI_DATATYPE_ALLOCATE(dmat_diag,(natom))
 call init_matlu(natom,nspinor,nsppol,paw_dmft%lpawu,level_diag)
 call init_matlu(natom,nspinor,nsppol,paw_dmft%lpawu,dmat_diag)
 do iatom=1,cryst_struc%natom
   lpawu=paw_dmft%lpawu(iatom)
   if(lpawu/=-1) then
     tndim=nspinor*(2*lpawu+1)
     do isppol=1,nsppol
       ABI_ALLOCATE(eigvectmatlu(iatom,isppol)%value,(tndim,tndim))
     end do
     !ABI_ALLOCATE(udens_atoms(iatom)%value,(2*(2*lpawu+1),2*(2*lpawu+1)))
     dmat_diag(iatom)%mat=czero
     level_diag(iatom)%mat=czero
   end if
 end do

! =================================================================
! Impose diago of density matrix
! =================================================================
 opt_diag=2 ! 2: diago density matrix

 if(hu(1)%jpawu_zero.and.nsppol==2) nsppol_imp=2 ! J=0 and nsppol=2
 if(.not.hu(1)%jpawu_zero.or.nsppol/=2) nsppol_imp=1  ! J/=0 ou nsppol=1
 if(opt_diag==1) then
! =================================================================
! Diagonalizes atomic levels
! =================================================================
   call init_oper(paw_dmft,energy_level,opt_ksloc=3)
   call compute_levels(cryst_struc,energy_level,hdc,pawang,paw_dmft)
   call diag_matlu(energy_level%matlu,level_diag,natom,&
&   prtopt=2,eigvectmatlu=eigvectmatlu,nsppol_imp=nsppol_imp)
   if(pawprtvol>=3) then
     write(message,'(a,2x,a,f13.5)') ch10,&
&     " == Print Diagonalized Energy levels for Fermi Level=",paw_dmft%fermie
     call wrtout(std_out,message,'COLL')
     call print_matlu(level_diag,natom,1)
   else
     write(message,'(a,2x,a,f13.5)') ch10,&
&     " == Energy levels Diagonalized for Fermi Level=",paw_dmft%fermie
   end if

 else if(opt_diag==2) then
! =================================================================
! Diagonalizes density matrix and keep eigenvectors in eigvectmatlu
! =================================================================
   if(pawprtvol>=3) then
     write(message,'(a,2x,a)') ch10,        " == Density Matrix ="
     call wrtout(std_out,message,'COLL')
     call print_matlu(green%occup%matlu,natom,1)
   end if

   write(message,'(2a,i3,13x,a)') ch10,'    ==  Diagonalization of density matrix == '
   call wrtout(std_out,message,'COLL')

!!  checkstop: we can have two different diagonalisation basis for the up and dn 
!!  but one use the same basis, unless the error is really to large(>0.1)
   call diag_matlu(green%occup%matlu,dmat_diag,natom,&
&   prtopt=4,eigvectmatlu=eigvectmatlu,nsppol_imp=nsppol_imp,checkstop=.false.)

   if(pawprtvol>=3) then
     write(message,'(a,2x,a)') ch10,&
&     " == Diagonalized Density Matrix in the basis used for QMC ="
     call wrtout(std_out,message,'COLL')

     call print_matlu(dmat_diag,natom,1)

     write(message,'(2a,i3,13x,a)') ch10,'    ==  Rotation of interaction matrix =='
     call wrtout(std_out,message,'COLL')
   end if
   
   if (.not.hu(1)%jpawu_zero) & 
   MSG_WARNING("In qmc_prep_ctqmc J/=0 and rotation matrix not rotated")
!  Rotate interaction.
!   call rotatevee_hu(cryst_struc,hu,nspinor,nsppol,pawprtvol,eigvectmatlu,udens_atoms)
!  diag weiss(1): weiss is not hermitian..
! call diag_matlu(weiss_for_rot%oper(1)%matlu,dmat_diag,natom,&
!&    prtopt=5,eigvectmatlu=eigvectmatlu)
! write(message,'(a,2x,a)') ch10,&
!&   " == Print Diagonalized Weiss field for first freq"
! call wrtout(std_out,message,'COLL')
! call print_matlu(dmat_diag,natom,1,opt_diag=1)
! print out
   if(nspinor==2.or.pawprtvol>=3) then
     write(message,'(a,2x,a,f13.5)') ch10,& ! debug
&    " == Print weiss for small freq before rot" ! debug
     call wrtout(std_out,message,'COLL') ! debug
     call print_matlu(weiss_for_rot%oper(1)%matlu,natom,1) !  debug
     write(message,'(a,2x,a,f13.5)') ch10,& ! debug
&    " == Print weiss for large freq before rot" ! debug
     call wrtout(std_out,message,'COLL') ! debug
     call print_matlu(weiss_for_rot%oper(paw_dmft%dmft_nwlo)%matlu,natom,1) !  debug
   end if

 else if(opt_diag==3) then
! =================================================================
! Diagonalizes weiss function
! =================================================================

   call diag_matlu(weiss_for_rot%oper(1)%matlu,dmat_diag,natom,&
&   prtopt=5,eigvectmatlu=eigvectmatlu,nsppol_imp=nsppol_imp)

   if(pawprtvol>=3) then
     write(message,'(a,2x,a,f13.5)') ch10,&
&     " == Print Diagonalized Weiss function for first frequency and Fermi level=",paw_dmft%fermie
     call wrtout(std_out,message,'COLL')

     call print_matlu(dmat_diag,natom,1)
   end if

 end if
 call flush_unit(std_out)

 if(opt_diag==2.or.opt_diag==3) then
!=======================================================
! Allocate, Compute, and Rotate atomic levels for CTQMC
!=======================================================
! ABI_DATATYPE_ALLOCATE(level_diag,(natom))
! call init_matlu(natom,nspinor,nsppol,paw_dmft%lpawu,level_diag)
! call zero_matlu(level_diag,natom)

   call init_oper(paw_dmft,energy_level,opt_ksloc=3)
   call compute_levels(cryst_struc,energy_level,hdc,pawang,paw_dmft)
   call rotate_matlu(energy_level%matlu,eigvectmatlu,natom,3,1)
   if(pawprtvol>=3) then
     write(message,'(a,2x,a,f13.5)') ch10," == Print Energy levels after rotation"
     call wrtout(std_out,message,'COLL')
!    call print_oper(energy_level,1,paw_dmft,1)
     call print_matlu(energy_level%matlu,natom,1)
   else 
     write(message,'(a,2x,a,f13.5)') ch10," == CT-QMC Energy levels rotated"
     call wrtout(std_out,message,'COLL')
   end if
 end if

 call flush_unit(std_out)

! call destroy_matlu(level_diag,natom)
! call nullify_matlu(level_diag,natom)
! ABI_DATATYPE_DEALLOCATE(level_diag)

! =================================================================
! Rotate weiss function according to eigenvectors.
! =================================================================
!!!stop
 write(message,'(a,2x,a)') ch10,&
& " == Rotation of weiss ="
 call wrtout(std_out,message,'COLL')
 do ifreq=1,paw_dmft%dmft_nwlo
   call rotate_matlu(weiss_for_rot%oper(ifreq)%matlu,eigvectmatlu,natom,3,1)
!  call checkdiag_matlu(weiss_for_rot%oper(ifreq)%matlu,natom,tol6)
 end do
 call flush_unit(std_out)
 if(pawprtvol>=3) then
   write(message,'(a,2x,a,f13.5)') ch10,& ! debug
&  " == Print weiss for small freq after rot" ! debug
   call wrtout(std_out,message,'COLL') ! debug
   call print_matlu(weiss_for_rot%oper(1)%matlu,natom,1) !  debug
   write(message,'(a,2x,a,f13.5)') ch10,&   ! debug
&  " == Print weiss for large freq after rot"   ! debug
   call wrtout(std_out,message,'COLL')   ! debug
   call print_matlu(weiss_for_rot%oper(paw_dmft%dmft_nwlo)%matlu,natom,1) ! debug
 end if
 call flush_unit(std_out)

! =================================================================
! Check if rotation is properly done.
! =================================================================
 if(3==4) then
   write(message,'(a,2x,a)') ch10,&
&   " == Print  dmat before rot"
   call wrtout(std_out,message,'COLL')
   call print_matlu(green%occup%matlu,natom,1)
   call rotate_matlu(green%occup%matlu,eigvectmatlu,natom,3,1)
   write(message,'(a,2x,a)') ch10,&
&   " == Print  dmat after rot"
   call wrtout(std_out,message,'COLL')
   call print_matlu(green%occup%matlu,natom,1)

   write(message,'(2a)') ch10,' QMC STOP: DEBUG'
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if
! =================================================================
! Check
! =================================================================

! write(message,'(a,2x,a,f13.5)') ch10,&
!&   " == Print weiss for small tau"
! call wrtout(std_out,message,'COLL')
! call print_matlu(weiss%oper(1)%matlu,natom,1)
! write(message,'(a,2x,a,f13.5)') ch10,&
!&   " == Print weiss for large tau"
! call wrtout(std_out,message,'COLL')
! call print_matlu(weiss%oper(paw_dmft%dmft_nwlo)%matlu,natom,1)
 call flush_unit(std_out)
 write(message,'(2a)') ch10,' Do not Check weiss_for_rot(last freq)'
 call wrtout(std_out,message,'COLL')
! call checkdiag_matlu(weiss_for_rot%oper(paw_dmft%dmft_nwlo)%matlu,natom,tol6,opt=nspinor)
 call flush_unit(std_out)
 write(message,'(2a)') ch10,' Do not Check weiss_for_rot(ifreq=1)'
 call wrtout(std_out,message,'COLL')
! call checkdiag_matlu(weiss_for_rot%oper(1)%matlu,natom,tol6,opt=nspinor)
 call flush_unit(std_out)

 master=0

! =================================================================
! Call ctqmc qmc with weiss
! =================================================================
 if(paw_dmft%dmft_prgn==1) then
   call print_green('Weiss_diag',weiss_for_rot,1,paw_dmft,pawprtvol=1,opt_wt=1,opt_decim=1)
 end if

 write(message,'(a,2x,a,f13.5)') ch10,&
& " == Preparing data for CTQMC"
 call wrtout(std_out,message,'COLL')

 if (pawprtvol>=3) then
   write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&  " == Print non diagonalized weiss function for small freq in the rotated basis"  ! debug
   call wrtout(std_out,message,'COLL')  ! debug
   call print_matlu(weiss_for_rot%oper(1)%matlu,natom,1)  ! debug
 end if

 do ifreq=1,paw_dmft%dmft_nwlo
   call inverse_oper(weiss_for_rot%oper(ifreq),option=1,prtopt=1)
 end do

 do iatom=1,cryst_struc%natom
   green%ecorr_qmc(iatom)=zero
   itypat=cryst_struc%typat(iatom)
   lpawu=paw_dmft%lpawu(iatom)
   tndim=2*lpawu+1
   if(lpawu/=-1) then
     nflavor=2*(tndim)

     !allocate(correl_loc(nflavor,nflavor))
     ABI_ALLOCATE(f_with_k,(paw_dmft%dmft_nwli,nflavor))
     ABI_ALLOCATE(fw1,(paw_dmft%dmft_nwlo))
     ABI_ALLOCATE(levels_ctqmc,(nflavor))
     !allocate(fw2(paw_dmft%dmft_nwli))
     iflavor=0
     do isppol=1,nsppol 
       do ispinor=1,nspinor
! < HACK >
         do ispinor2=1,nspinor
! </ HACK >
           do im=1,tndim
! < HACK >
             do im2=1,tndim
               if ( im .eq. im2 .and. ispinor.eq.ispinor2) then
! </ HACK >
!           iflavor=im+(isppol-1)*tndim
                 iflavor=iflavor+1
!            Do spline of weiss function for im and isppol
!            Construction of fw1
!           =================================================================
                 do ifreq=1,paw_dmft%dmft_nwlo
!             fw1(ifreq)= weiss_for_rot%oper(ifreq)%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor) 
! < HACK >
  ! Just copy G0 in Fw1 (diagonal part only)
!             retemp=real(fw1(ifreq))/(real(fw1(ifreq))**2+imag(fw1(ifreq))**2)
!             imtemp=imag(fw1(ifreq))/(real(fw1(ifreq))**2+imag(fw1(ifreq))**2)+paw_dmft%omega_lo(ifreq)
!             fw1(ifreq)=cmplx(-retemp,imtemp,kind=dp)
                   fw1(ifreq)= -weiss_for_rot%oper(ifreq)%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor) &
                   + cmplx(zero,paw_dmft%omega_lo(ifreq),kind=dp)
! </ HACK >
                 end do
!            Performing Spline
!           =================================================================
! < HACK >
!  Spline fw1 to linear frequencies
! nothing modified
                 call spline_fct(fw1,f_with_k(:,iflavor),-1,paw_dmft)
! </ HACK >


                 levels_ctqmc(iflavor)=real(energy_level%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor),kind=dp)

                 if(nsppol==1.and.nspinor==1) then
                   f_with_k(:,iflavor+tndim)=f_with_k(:,iflavor)
                   levels_ctqmc(iflavor+tndim)=levels_ctqmc(iflavor)
                 end if
! < HACK >
               else
                 do ifreq=1,paw_dmft%dmft_nwlo
                   weiss_for_rot%oper(ifreq)%matlu(iatom)%mat(im,im2,isppol,ispinor,ispinor2) = czero
                 end do
               end if
! </ HACK >
             end do !im2
           end do !im
         end do  !ispinor2
       end do  !ispinor
     end do  !isppol
! < HACK >
     if(pawprtvol>=3) then
       write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&      " == Print diagonalized inverse weiss function for small freq in the rotated basis"  ! debug
       call wrtout(std_out,message,'COLL')  ! debug
       call print_matlu(weiss_for_rot%oper(1)%matlu,natom,1)  ! debug
       !write(message,'(i3,4x,2e21.14)') 1,weiss_for_rot%oper(1)%matlu(1)%mat(1,1,1,1,1)
       !call wrtout(std_out,message,'COLL')  ! debug
     end if
   ! </ HACK >
     ABI_DEALLOCATE(fw1)


! =================================================================
!    Main call to CTQMC code
! =================================================================
     write(message,'(a,2x,a)') ch10,&
&    " == Initializing CTQMC"
     call wrtout(std_out,message,'COLL')

     call CtqmcInterface_init(hybrid,paw_dmft%dmftqmc_seed,paw_dmft%dmftqmc_n, &
&         paw_dmft%dmftqmc_therm, paw_dmft%dmftctqmc_meas,nflavor,paw_dmft%dmftqmc_l,one/paw_dmft%temp,zero,&
&         std_out,paw_dmft%spacecomm)

     call CtqmcInterface_setOpts(hybrid,&
                                     opt_Fk      =1,& 
&                                    opt_order   =paw_dmft%dmftctqmc_order ,&
&                                    opt_movie   =paw_dmft%dmftctqmc_mov   ,&
&                                    opt_analysis=paw_dmft%dmftctqmc_correl,&
&                                    opt_check   =paw_dmft%dmftctqmc_check ,&
&                                    opt_noise   =paw_dmft%dmftctqmc_grnns ,&
&                                    opt_spectra =paw_dmft%dmftctqmc_mrka  ,&
&                                    opt_gmove   =paw_dmft%dmftctqmc_gmove )  
     write(message,'(a,2x,2a)') ch10,&
&    " == Initialization CTQMC done", ch10
     call wrtout(std_out,message,'COLL')

     ABI_ALLOCATE(gw_tmp,(paw_dmft%dmft_nwlo,nflavor+1))

!     use  gw_tmp to put freq
     do ifreq=1,paw_dmft%dmft_nwlo
       gw_tmp(ifreq,nflavor+1)=cmplx(zero,paw_dmft%omega_lo(ifreq),kind=dp)
     end do

     ABI_ALLOCATE(gtmp,(paw_dmft%dmftqmc_l,nflavor))
     call CtqmcInterface_run(hybrid,f_with_k(1:paw_dmft%dmftqmc_l,:),Gtau=gtmp,&
&     Gw=gw_tmp,D=Doccsum,E=green%ecorr_qmc(iatom),&
&     Noise=Noise,matU=hu(itypat)%udens,opt_levels=levels_ctqmc)

! </ HACK >
!! < HACK >
!     iflavor=0
!     do isppol=1,nsppol 
!       do ispinor=1,nspinor
!         do im=1,tndim
!           do ifreq=1,paw_dmft%dmft_nwlo
!             iflavor = iflavor + 1
!             gw_loc%oper(ifreq)%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor)=gw_tmp(ifreq,iflavor)
!           enddo !ifreq
!         enddo !im
!       enddo !ispinor
!     enddo !isppol
!     write(message,'(2a,i3,13x,a)') ch10,'   ===  Integrate local part of G(w)ctqmc'
!     call wrtout(std_out,message,'COLL')
!     call integrate_green(cryst_struc,gw_loc,mpi_enreg,paw_dmft,&
!&     pawang,prtopt=2,opt_ksloc=2,opt_after_solver=1)
!     call printocc_green(gw_loc,5,paw_dmft,2,chtype="gw_loc")
!! </ HACK >
     !----------------------------------------
     ! <DEBUG>
     !----------------------------------------
     ! Construct UNIT
     if(paw_dmft%idmftloop < 10) then
       write(gtau_iter,'("0",i1)') paw_dmft%idmftloop
     elseif(paw_dmft%idmftloop >= 10 .and. paw_dmft%idmftloop < 100) then
       write(gtau_iter,'(i2)') paw_dmft%idmftloop
     else
       gtau_iter="xx"
     end if
     if(iatom < 10) then
       write(iatomnb,'("0",i1)') iatom
     elseif(iatom >= 10 .and. iatom < 100) then
       write(iatomnb,'(i2)') iatom
     else
       iatomnb='xx'
     end if

     if(myproc .eq. mod(nproc+1,nproc)) then
! < HACK >
       open(unit=4242, file=trim(paw_dmft%filapp)//"_atom_"//iatomnb//"_Gtau_"//gtau_iter//".dat")
       call Ctqmc_printGreen(hybrid%hybrid,4242)
       close(4242)
       open(unit=4243, file=trim(paw_dmft%filapp)//"_atom_"//iatomnb//"_F_"//gtau_iter//".dat")
       call BathOperator_printF(hybrid%hybrid%bath,4243)
       close(4243)
       !open(unit=4242, file=trim(paw_dmft%filapp)//"_atom_"//iatomnb//"_Gw_"//gtau_iter//".dat")
       !do ifreq=1,paw_dmft%dmft_nwlo
       !  write(4242,'(6f21.14)') (/ (gw_tmp(ifreq,iflavor), iflavor=1, nflavor) /) 
       !end do
       !close(4242)
! </ HACK >
     end if
     !----------------------------------------
     ! </DEBUG>
     !----------------------------------------
     write(message,'(a,2x,a)') ch10,&
&    " == Destroy CTQMC"
     call wrtout(std_out,message,'COLL')
     call CtqmcInterface_finalize(hybrid)
     write(message,'(a,2x,a)') ch10,&
&    " == Destroy CTQMC done"
     call wrtout(std_out,message,'COLL')
     ABI_DEALLOCATE(f_with_k)
     ABI_DEALLOCATE(levels_ctqmc)
   !write(message,'(i3,4x,2e21.14)') 4,weiss_for_rot%oper(1)%matlu(1)%mat(1,1,1,1,1)
   !call wrtout(std_out,message,'COLL')  ! debug
     ! FIXME green%ecorr_qmc and gtmp2 are already reduced inside the ctqmc.....
     !do itau=1,paw_dmft%dmftqmc_l
     !  !gtmp(ifreq,:)=gtmp2(ifreq,:)/nproc
     !  gtmp(itau,:)=gtmp2(itau,:)
     !enddo
     !deallocate(gtmp2)


     do itau=1,paw_dmft%dmftqmc_l
       green%oper_tau(itau)%matlu(iatom)%mat(:,:,:,:,:)=czero
     end do
     green%occup_tau%matlu(iatom)%mat(:,:,:,:,:)=czero

     do ifreq=1,paw_dmft%dmft_nwlo
       green%oper(ifreq)%matlu(iatom)%mat(:,:,:,:,:)=czero
     end do
     green%occup%matlu(iatom)%mat(:,:,:,:,:)=czero

     iflavor=0
     do isppol=1,nsppol
       do ispinor=1,nspinor
         do im=1,tndim
              ! NNtodo here: nsppol=1 above, only, with symetrisation
              ! automatic
!           iflavor=im+(isppol-1)*tndim
           iflavor=iflavor+1
           do itau=1,paw_dmft%dmftqmc_l
             green%oper_tau(itau)%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor)=gtmp(itau,iflavor)
             if(nsppol==1.and.nspinor==1) then
               green%oper_tau(itau)%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor)=&
&               (gtmp(itau,iflavor)+gtmp(itau,iflavor+tndim))/two
             end if
              ! NNtodo here: isppol above should be one and symetrized
              ! gtmp
           end do
!           ifreq2=0
           do ifreq=1,paw_dmft%dmft_nwlo
!             if(paw_dmft%select_log(ifreq)==1) then
!               ifreq2=ifreq2+1
             green%oper(ifreq)%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor)=gw_tmp(ifreq,iflavor)
             if(nsppol==1.and.nspinor==1) then
               green%oper(ifreq)%matlu(iatom)%mat(im,im,isppol,ispinor,ispinor)=&
&               (gw_tmp(ifreq,iflavor)+gw_tmp(ifreq,iflavor+tndim))/two
             end if
              ! NNtodo here: isppol above should be one and symetrized
              ! gw_tmp
!             endif
           end do
         end do
       end do
     end do
     ABI_DEALLOCATE(gw_tmp)
     ABI_DEALLOCATE(gtmp)
     if(nsppol==1.and.nspinor==1) then
       write(message,'(a,2x,a,f13.5)') ch10,& 
&       " == nsppol==1 and nspden==1: Green functions from CTQMC have been symetrized over spin"
       call wrtout(std_out,message,'COLL')  
     end if
     !write(message,'(i3,4x,2e21.14)') 5,weiss_for_rot%oper(1)%matlu(1)%mat(1,1,1,1,1)
     !call wrtout(std_out,message,'COLL')  ! debug
!     do im=1,tndim
!       do itau=1,paw_dmft%dmftqmc_l
!         gtt=(green%oper_tau(itau)%matlu(iatom)%mat(im,im,1,1,1)+&
!&         green%oper_tau(itau)%matlu(iatom)%mat(im,im,2,1,1))/two
!         green%oper_tau(itau)%matlu(iatom)%mat(im,im,1,1,1)=gtt
!         green%oper_tau(itau)%matlu(iatom)%mat(im,im,2,1,1)=gtt
!       enddo
!     enddo
!     write(6,*)" SYMETRISATION OVER ISPPOL"
     !deallocate(correl_loc,f_with_k,gtmp,fw1,fw2) 
   end if
 end do ! iatom


 if(paw_dmft%dmft_prgn==1) then
   call print_green('QMC_diag_notsym',green,1,paw_dmft,pawprtvol=1,opt_wt=2)
   call print_green('QMC_diag_notsym',green,1,paw_dmft,pawprtvol=1,opt_wt=1)
 end if
 !write(message,'(i3,4x,2e21.14)') 6,weiss_for_rot%oper(1)%matlu(1)%mat(1,1,1,1,1)
 !call wrtout(std_out,message,'COLL')  ! debug
! < HACK >
! =================================================================
! Inverse Weiss, then
! Copy Weiss_for_rot into weiss and rotate back weiss to the original basis
! =================================================================
 do ifreq=1,paw_dmft%dmft_nwlo
   call inverse_oper(weiss_for_rot%oper(ifreq),option=1,prtopt=1)
   call copy_matlu(weiss_for_rot%oper(ifreq)%matlu,weiss%oper(ifreq)%matlu,natom)
   call rotate_matlu(weiss%oper(ifreq)%matlu,eigvectmatlu,natom,3,0)
 end do
! </ HACK >
! =================================================================
! Here compute Self energy from Dyson and print it
! Warning : Weiss_for_rot is inversed inside dyson
! =================================================================
 call initialize_self(self,paw_dmft)
 call dyson(green,paw_dmft,self,weiss_for_rot,opt_weissself=2)
! call rw_self(self,mpi_enreg,paw_dmft,prtopt=2,opt_rw=2,opt_char="diag")
 call destroy_self(self)
 !write(message,'(i3,4x,2e21.14)') 7,weiss%oper(1)%matlu(1)%mat(1,1,1,1,1)
 !call wrtout(std_out,message,'COLL')  ! debug

! =================================================================
! Rotate back green function to original basis (non-diagonal)
!  (and Weiss for further use: might be useful if an back Fourier
!     transformation is done).  
! =================================================================
 if(pawprtvol>=3) then
   write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&  " == Print green function for small tau after CTQMC"  ! debug
   call wrtout(std_out,message,'COLL')  ! debug
   call print_matlu(green%oper_tau(1)%matlu,natom,1)  ! debug
   write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&  " == Print green function for small freq after CTQMC"  ! debug
   call wrtout(std_out,message,'COLL')  ! debug
   call print_matlu(green%oper(1)%matlu,natom,1)  ! debug
 end if

 write(message,'(a,2x,a,f13.5)') ch10,&  
& " == Rotate Green function to original basis "
 call wrtout(std_out,message,'COLL') 
 !write(message,'(i3,4x,2e21.14)') 8,weiss%oper(1)%matlu(1)%mat(1,1,1,1,1)
 !call wrtout(std_out,message,'COLL')  ! debug
 do itau=1,paw_dmft%dmftqmc_l
   call rotate_matlu(green%oper_tau(itau)%matlu,eigvectmatlu,natom,3,0)
 end do
 !write(message,'(i3,4x,2e21.14)') 9,weiss%oper(1)%matlu(1)%mat(1,1,1,1,1)
 !call wrtout(std_out,message,'COLL')  ! debug
 do ifreq=1,paw_dmft%dmft_nwlo
   call rotate_matlu(green%oper(ifreq)%matlu,eigvectmatlu,natom,3,0)
 end do
 !write(message,'(i3,4x,2e21.14)') 10,weiss%oper(1)%matlu(1)%mat(1,1,1,1,1)
 !call wrtout(std_out,message,'COLL')  ! debug

 if(pawprtvol>=3) then
   write(message,'(a,2x,a,f13.5)') ch10,&                  ! debug
&  " == Print green function for small time after rotation (in the original basis)" ! debug
   call wrtout(std_out,message,'COLL')                  ! debug
   call print_matlu(green%oper_tau(1)%matlu,natom,1)  ! debug
   write(message,'(a,2x,a,f13.5)') ch10,&                  ! debug
&  " == Print green function for small freq after rotation (in the original basis)" ! debug
   call wrtout(std_out,message,'COLL')                  ! debug
   call print_matlu(green%oper(1)%matlu,natom,1)  ! debug
   !< HACK >
   write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&  " == Print diagonalized weiss_for_rot function after rotation for small freq in the original basis"  ! debug
   call wrtout(std_out,message,'COLL')  ! debug
   call print_matlu(weiss_for_rot%oper(1)%matlu,natom,1)  ! debug
   !</ HACK >
   write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&  " == Print weiss function for small freq in the original basis"  ! debug
   call wrtout(std_out,message,'COLL')  ! debug
   call print_matlu(weiss%oper(1)%matlu,natom,1)  ! debug
 end if

!  === Compute Unsymetrized Occupations 
 call occup_green_tau(green)

 
 ABI_DATATYPE_ALLOCATE(matlu1,(natom))
 call init_matlu(natom,nspinor,nsppol,paw_dmft%lpawu,matlu1)
 call copy_matlu(green%occup_tau%matlu,matlu1,natom)
 call sym_matlu(cryst_struc,matlu1,pawang)

 write(message,'(a,2x,a,f13.5)') ch10," == Occupation from G(tau)" 
 call wrtout(std_out,message,'COLL')
 call print_matlu(green%occup_tau%matlu,natom,1)

 write(message,'(a,2x,a,f13.5)') ch10," == Symetrized occupations"
 call wrtout(std_out,message,'COLL')
 call print_matlu(matlu1,natom,1)

 call diff_matlu("CTQMC Occup","CTQMC Occup symetrized",green%occup_tau%matlu,matlu1,natom,0,tol4,ierr)
 call destroy_matlu(matlu1,natom)
 call nullify_matlu(matlu1,natom)
 ABI_DATATYPE_DEALLOCATE(matlu1)

! =================================================================
! Symetrise green function G(tau) to recover symetry 
! artificially broken by QMC
! =================================================================
 write(message,'(a,2x,a,f13.5)') ch10,&  
& " == Symetrise green function after QMC "
 call wrtout(std_out,message,'COLL')  
 do itau=1,paw_dmft%dmftqmc_l
   call sym_matlu(cryst_struc,green%oper_tau(itau)%matlu,pawang)
 end do
 do ifreq=1,paw_dmft%dmft_nwlo
   call sym_matlu(cryst_struc,green%oper(ifreq)%matlu,pawang)
 end do
 if(pawprtvol>=3) then
   write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&  " == Print green function for small time after symetrisation"  !  debug
   call wrtout(std_out,message,'COLL')  ! debug
   call print_matlu(green%oper_tau(1)%matlu,natom,1)  ! debug
   write(message,'(a,2x,a,f13.5)') ch10,&  ! debug
&  " == Print green function for small freq after symetrisation"  !  debug
   call wrtout(std_out,message,'COLL')  ! debug
   call print_matlu(green%oper(1)%matlu,natom,1)  ! debug
 end if
 if(paw_dmft%dmft_prgn==1) then
   call print_green('QMC_sym',green,1,paw_dmft,pawprtvol=1,opt_wt=2)
   call print_green('QMC_sym',green,1,paw_dmft,pawprtvol=1,opt_wt=1)
 end if

!  === Compute Occupations  (Symetrized from oper_tau)
 call occup_green_tau(green)


!  === Print occupations
 call printocc_green(green,6,paw_dmft,3)

 call destroy_oper(energy_level)
 call destroy_matlu(dmat_diag,natom)
 call nullify_matlu(dmat_diag,natom)
 ABI_DATATYPE_DEALLOCATE(dmat_diag)
 call destroy_matlu(level_diag,natom)
 call nullify_matlu(level_diag,natom)
 ABI_DATATYPE_DEALLOCATE(level_diag)
 do iatom=1,cryst_struc%natom
   lpawu=paw_dmft%lpawu(iatom)
   if(lpawu/=-1) then
     do isppol=1,nsppol
       ABI_DEALLOCATE(eigvectmatlu(iatom,isppol)%value)
       !ABI_DEALLOCATE(udens_atoms(iatom))
     end do
   end if
 end do
 !ABI_DATATYPE_DEALLOCATE(udens_atoms)
 ABI_DATATYPE_DEALLOCATE(eigvectmatlu)
 call destroy_green(weiss_for_rot)
 call destroy_green(gw_loc)

end subroutine qmc_prep_ctqmc
!!***

!!***
