# -*- Autoconf -*-
#
# Copyright (C) 2013-2014 ABINIT Group (Yann Pouillon)
#
# This file is part of the ABINIT software package. For license information,
# please see the COPYING file in the top-level directory of the ABINIT source
# distribution.
#

#
# MPI support for the ABINIT Test Suite
#



# _AX_MPIRUN_IFELSE(PROGRAM, [ACTION-IF-TRUE], [ACTION-IF-FALSE])
# ---------------------------------------------------------------
#
# Compile, link, and run in parallel.
# This macro can be used during the selection of a compiler.
# We also remove conftest.o as if the compilation fails, some compilers
# don't remove it.  We remove gmon.out and bb.out, which may be
# created during the run if the program is built with profiling support.
#
m4_define([_AX_MPIRUN_IFELSE],[
  m4_ifvaln([$1], [AC_LANG_CONFTEST([$1])])dnl
  rm -f conftest$ac_exeext
  test "${MPI_2PROCS}" = "" && MPI_2PROCS="-np 2"
  AS_IF([_AC_DO_VAR(ac_link) && _AC_DO_TOKENS(${MPI_RUNNER} ${MPI_2PROCS} conftest$ac_exeext) >&AS_MESSAGE_LOG_FD],
    [$2],
    [AS_ECHO(["$as_me: program exited with status $ac_status"]) >&AS_MESSAGE_LOG_FD
  _AC_MSG_LOG_CONFTEST
  m4_ifvaln([$3],
    [( exit $ac_status )
  $3])dnl])[]dnl
  rm -rf conftest.dSYM
  rm -f core *.core core.conftest.* gmon.out bb.out conftest$ac_exeext conftest.$ac_objext m4_ifval([$1],
    [conftest.$ac_ext])[]dnl
])# _AX_MPIRUN_IFELSE



# AX_MPIRUN_IFELSE(PROGRAM,
#                  [ACTION-IF-TRUE], [ACTION-IF-FALSE],
#                  [ACTION-IF-CROSS-COMPILING = RUNTIME-ERROR])
# -------------------------------------------------------------
#
# Compile, link, and run in parallel. Requires that the compiler for the
# current language was checked for, hence do not use this macro in macros
# looking for a compiler.
#
AC_DEFUN([AX_MPIRUN_IFELSE],
[AC_LANG_COMPILER_REQUIRE()dnl
m4_ifval([$4], [],
  [AC_DIAGNOSE([cross],
    [$0 called without default to allow cross compiling])])dnl
  AS_IF([test "$cross_compiling" = yes],
    [m4_default([$4],
      [AC_MSG_FAILURE([cannot run test program while cross compiling])])],
      [_AX_MPIRUN_IFELSE($@)])
]) # AX_MPIRUN_IFELSE



                    ########################################



# ABITEST_MPI_INIT()
# ------------------
#
# Looks for a MPI runner using the provided prefix.
# Note 1: this is a convenience feature, purely for comfort.
# Note 2: should be run as early as possible.
#
AC_DEFUN([ABITEST_MPI_INIT],[
  dnl Init
  abi_mpi_has_runner="no"
  abi_mpi_runner_ok="unknown"
  test "${MPI_RUNNER}" = "" && MPI_RUNNER="${with_mpi_runner}"

  if test "${enable_mpi}" = "yes"; then

    if test "${with_mpi_prefix}" != ""; then

      AC_MSG_NOTICE([using MPI prefix '${with_mpi_prefix}'])

      dnl Look for a runner
      AC_MSG_CHECKING([for a MPI runner])
      if test -x "${with_mpi_prefix}/bin/mpirun"; then
        abi_mpi_has_runner="yes"
        MPI_RUNNER="${with_mpi_prefix}/bin/mpirun"
      elif test -x "${with_mpi_prefix}/bin/mpiexec"; then
        abi_mpi_has_runner="yes"
        MPI_RUNNER="${with_mpi_prefix}/bin/mpiexec"
      elif test -x "${with_mpi_prefix}/bin/dmpirun"; then
        abi_mpi_has_runner="yes"
        MPI_RUNNER="${with_mpi_prefix}/bin/dmpirun"
      fi
      if test "${MPI_RUNNER}" = ""; then
        AC_MSG_RESULT([none found])
        abi_mpi_runner_ok="no"
      else
        AC_MSG_RESULT([${MPI_RUNNER}])
        abi_mpi_runner_ok="yes"
      fi

    else dnl with_mpi_prefix

      dnl Look for a MPI runner if necessary
      if test "${MPI_RUNNER}" = ""; then
        AC_CHECK_PROGS(MPI_RUNNER,[mpirun mpiexec dmpirun])
      else
        if test -x "${MPI_RUNNER}"; then
          abi_mpi_runner_ok="yes"
        else
          unset abi_mpi_runner_ok
          AC_CHECK_PROG(abi_mpi_runner_ok,${MPI_RUNNER},yes,no)
        fi
        if test "${abi_mpi_runner_ok}" = "no"; then
          AC_MSG_WARN([invalid MPI runner: ${MPI_RUNNER}])
          AC_MSG_WARN([you might not be able to run the test suite])
        fi
      fi

    fi dnl with_mpi_prefix

  else dnl enable_mpi

    AC_MSG_NOTICE([MPI support disabled])
    MPI_RUNNER=""
    MPI_2PROCS=""

  fi dnl enable_mpi

  dnl Enable substitution
  AC_SUBST(MPI_RUNNER)
]) # ABITEST_MPI_INIT
